#line 291 "commands.u"

#include <stdlib.h>
#include <stdint.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <stdarg.h>
#include "words.h"

#line 7 "file.u"

typedef struct {
  char *name;
  char *error;
  size_t size;
  unsigned char* contents;
} file_t;
#line 22 "commands.u"

typedef struct {
  void* func;
  void* parse_table[200];
} command_t;

#line 124 "commands.u"

#define PARSER_FUNC(FN)  \
int FN(void ***ptable, char **input, void*** ostack) { \
   void **PT= *ptable; \
   void **OS= *ostack; \
   char *IN= *input; 

#define PF_EPILOGUE \
done: \
   *ptable= PT; \
   *ostack= OS; \
   *input= IN; \
   return 0; }
#line 41 "t2c-ar.c"

void prt(char *tm, ...);
void prtv(char *tm, va_list ap);
void cmd_setcmode(char **V) ;
void cmd_sethmode(char **V) ;
void cmd_setbmode(char **V) ;
static int isidbegin(unsigned int chr);
static int isid(unsigned int chr);
static int skip_quote(char **inp);
static char* collect_str(char *b,char *e);
void parse_string(char* str);
 
void cmd_stack(char **V);
void cmd_tree(char **V);
static int skip_paren(char **s);
void cmd_qsort(char **V);
static char *gensymbol();
static void cmd_buffer(char **V);
void cmd_for_words(char **V);
void cmd_unify(char **V);
static char *get_next_section(char *start,char *secname);
static void print_help_toc();
static void do_help(int argc,char **argv);
static void print_man(char *name);
static char *get_section(char *name);
static void cmd_splitwords(char **V);
static void cmd_fileread(char **V);
#line 106 "commands.u"

PARSER_FUNC(fID) 
   char *expect= *(PT++), *b;
   while(isspace(*IN)) IN++;
   if (!*IN) return 1;
   if (!isidbegin(*IN)) return 1;
   b= IN;
   while(isid(*IN)) IN++;
   if (expect)
     if (memcmp(b, expect, IN-b)) return 1;
     else goto done;
   *(OS++)= collect_str(b,IN);
PF_EPILOGUE
#line 153 "commands.u"

PARSER_FUNC(fPUN)
  char *expect= *(PT++);
  while(isspace(*IN)) IN++;
  while(*expect && *IN==*expect) { IN++; expect++; }
  if (*expect) return 1;
PF_EPILOGUE

PARSER_FUNC(fROUND_ANY)
  while(isspace(*IN)) IN++;
  if (*IN!='(') return 1;
  char *b= ++IN;
  int k= 1;
  while(1) {
    switch(*IN) {
     case 0: return 1;
     case '(': k++; break;
     case ')':
       if (--k==0) { *(OS++)= collect_str(b,IN++); goto done; }
       break;
     case '\'': 
     case '\"': if (skip_quote(&IN)) return 1; break;
     default: IN++;
    }
  }
PF_EPILOGUE

PARSER_FUNC(fCURLY_ANY)
  while(isspace(*IN)) IN++;
  if (*IN!='{') return 1;
  char *b= ++IN;
  int k= 1;
  while(1) {
    switch(*IN) {
     case 0: return 1;
     case '{': k++; IN++; break;
     case '}':
       if (--k==0) { *(OS++)= collect_str(b,IN++); goto done; }
       IN++;
       break;
     case '\'': 
     case '\"':
       if (skip_quote(&IN)) return 1; break;
     default: IN++;
    }
  }
PF_EPILOGUE

#line 459 "tree.u"

PARSER_FUNC(fROUND_ARGS) 
  int nargs= (intptr_t) *(PT++);
  while(isspace(*IN)) IN++;
  if (*IN!='(') return 1;
  char *b= ++IN;
  int got_args= 0;
  int done= 0;
  char *k;

  goto stupid_macro_stuff;

no_good:
  while(got_args--) free(*(--OS));
  return 1;

stupid_macro_stuff:
  while(!done) {
    switch(*IN) {
     case 0: case ']': case '}':
	goto no_good;
     case '\'': 
     case '\"': if (skip_quote(&IN)) goto no_good; break;
     case '(': case '[': case '{':
       if (skip_paren(&IN)) goto no_good;
       break;
     case ')':
        for(k=b;k<IN;k++) if (!isspace(*k)) break;
        if (k!=IN) {
          *(OS++)= collect_str(b, IN);
	  got_args++; 
        }
	done= 1;
	IN++;
        break;
     case ',':
        *(OS++)= collect_str(b, IN++); b= IN; got_args++;
        break;
     default: IN++;
    }
  }
  if (got_args!=nargs) goto no_good;
PF_EPILOGUE

#line 38 "qsort.u"

PARSER_FUNC(fANY) 

   int (*next_function)(void ***, char **, void*** );
   char *b, *e;
   char *sub_input;
   void **stack_slot;
   int found= 0;
   next_function= *(PT++);
   stack_slot= OS++;
 
   b= IN;
   for(e=IN;e[0];e++)
   {
      char *sub_input;
      void **sub_pt;
      void **sub_os;
      sub_input= e;
      sub_pt= PT;
      sub_os= OS;
      if (!(*next_function)(&sub_pt, &sub_input, &sub_os))
      {
         OS= sub_os;
         PT= sub_pt;
         IN= sub_input;
         *stack_slot= collect_str(b, e);
         found= 1;
         break;
      }
      if (e[0]=='\'' || e[0]=='\"') {
         if (skip_quote(&e)) return 1;
         continue;
      }
      if (e[0]=='(' || e[0]=='[' || e[0]=='{') { 
         if (skip_paren(&e)) return 1;
         e-= 1;
         continue;
      }
   }
   if (!found) return 1;
PF_EPILOGUE


#line 42 "commands.u"

static int omode = 'C'+'H';

#line 11 "help.u"

static char help[]=
#line 227 "t2c-ar.c"
"\n"
"[[ tree ]]\n"
"\n"
"The tree struct stores a collection of objects inside a balanced tree. The\n"
"syntax is:\n"
"\n"
"  TREE : tree(KEY,DATA)\n"
"   access { ACCESS }\n"
"   compare { COMPARE }\n"
"\n"
"The type DATA is the type of the objects to be stored. KEY is the type of\n"
"the key field in these objects. The ACCESS code shall store the key associated\n"
"with the \047DATA\047 typed value \047data\047 in the variable \047key\047. For example:\n"
"\n"
"   key= data -> employee_id;\n"
"\n"
"where\n"
"\n"
"  DATA = struct { int employee_id; char *name; }\n"
"  KEY  = int\n"
"\n"
"The COMPARE code shall compare key1 to key2, which are of type KEY. For this\n"
"example:\n"
"\n"
"  result = key1 - key2;\n"
"\n"
"The result of the comparision shall be stored in the variable \047result\047, which\n"
"is of int type. So, a full example would look like:\n"
"\n"
"  employee_by_id_t : tree(int,employee_t)\n"
"     access { key= data->employee_id; }\n"
"     compare { result= key1-key2; }\n"
"\n"
"The generated functions and types are as follows:\n"
"\n"
"  typedef struct TREE_s TREE;\n"
"\n"
"  TREE *TREE_new();\n"
"  int PFX_find(TREE *T, KEY key, DATA *value);\n"
"  int PFX_insert(TREE *T, DATA data) ;\n"
"  int PFX_remove(TREE *T, KEY key, DATA *value);\n"
"  int PFX_destroy(TREE *T,DATA *value);\n"
"\n"
"Here, PFX is the same as TREE, but the trailing \047_t\047 is removed from PFX\n"
"if there is one. i.e. For the above example, we would have\n"
"\n"
"  PFX= employee_by_id\n"
"\n"
"Anyway, PFX_find finds the given key in the tree, if it\047s there 1 is returned\n"
"and the associated value is stored in \047*value\047 if \047value\047 is not null.\n"
"\n"
"PFX_insert returns zero if a value with the same key is already in the tree.\n"
"Otherwise it inserts the given value and returns one.\n"
"\n"
"PFX_remove returns zero if the given key is not found in the tree. Otherwise,\n"
"one is returned and the removed value is stored in \047*value\047 if \047value\047 is not\n"
"null.\n"
"\n"
"PFX_destroy returns zero if the tree is empty. Otherwise, the least element\n"
"is removed from the tree and stored in \047*value\047 if \047value\047 is not null. The\n"
"return value is 1 in this case. This is useful for deconstructing a tree:\n"
"\n"
"  while(employee_by_id_destroy(employees, &person))\n"
"    free(person);\n"
"  free(employees);\n"
"\n"
"[[ buffer ]]\n"
"\n"
"The buffer command creates a type and associated functions for a buffer \n"
"object. Usage is:\n"
"\n"
" TYPE: buffer(SUBTYPE,BLOCKSIZE);\n"
"\n"
"Here, SUBTYPE is the type of the objects to be stored in the buffer.\n"
"BLOCKSIZE is the size of each block for the buffer. The buffers are\n"
"allocated in BLOCKSIZE increments.  The generated functions are:\n"
"\n"
"  TYPE*    PFX_new();\n"
"  void     PFX_append(TYPE* buffer, SUBTYPE data);\n"
"  SUBTYPE* PFX_collect(TYPE *buffer, int *len);\n"
"  SUBTYPE* PFX_collectd(TYPE *buffer, int *len);\n"
"              Same as collect, but also calls PFX_free() on the buffer.\n"
"  void     PFX_free(TYPE *buffer);\n"
"\n"
"Here, PFX is equal to TYPE, but the trailing _t is cut off if there is one.\n"
"So, if you have:\n"
"\n"
"  intbuffer_t : buffer(int,50);\n"
"\n"
"You would get functions like: intbuffer_new(), intbuffer_collect(). etc.\n"
"\n"
"[[ for_words ]]\n"
"\n"
"This command expands the given text for each word. It loops over the list\n"
"of words using the loop variable and prints the text, replacing the\n"
"loop variable with the current word. The syntax is:\n"
"\n"
" for_words LOOP_VAR (WORD WORD ...)\n"
" {\n"
"   TEXT\n"
" }\n"
"\n"
"Words are separated by whitespace characters. Here is an example:\n"
"\n"
" for_words X (a b c)\n"
" {\n"
"   foo(X);\n"
" }\n"
"\n"
"The output would be:\n"
"\n"
" foo(a);\n"
" foo(b);\n"
" foo(c);\n"
"[[ split_words ]]\n"
"\n"
"This command generates the split_words function whose prototype is:\n"
"\n"
"  char **split_words(char *src);\n"
"\n"
"It splits src into words, using whitespace characters as the delimiter\n"
"and returns them as a null-terminated array. The syntax for this command is:\n"
"\n"
"  import split_words as SCOPE;\n"
"\n"
"where SCOPE is either \047public\047 or \047private\047, which declare the function\n"
"as extern or static respectively.\n"
"\n"
"The returned array of strings is just one memory block which contain\n"
"both the strings and the pointers. Just calling free() on the returned\n"
"array frees everything. If you want to store one of the words, just\n"
"use strdup().\n"
"\n"
"All whitespace characters are eaten up by this function unless they\n"
"occur within double quote pairs. If something is given in double quotes,\n"
"then the whole double quoted text is taken to be a separate word.\n"
"[[ file_read ]]\n"
"\n"
"This command generates the following functions:\n"
"\n"
"  file_t *file_read(char *name);\n"
"  file_t *file_read_descriptor(FILE *D,char *name);\n"
"\n"
"These functions read the given file to EOF and store the contents in the\n"
"returned file_t. The file_t struct contains the following:\n"
" \n"
"  typedef struct {\n"
"    char *name;\n"
"    char *error;\n"
"    size_t size;\n"
"    unsigned char* contents;\n"
"  } file_t;\n"
"\n"
"Both of these functions allocate contents to be as big as size+1. The last\n"
"byte is used to store a null in it, in case you want to consider the whole\n"
"file as a string. The name, error and contents fields must be freed before\n"
"you free the file_t.\n"
"\n"
"If there are no errors, the error field will be null. Otherwise, the contents\n"
"field will be null and the error field will contain the strerror() of the\n"
"errno returned from fread. Here is the syntax for using this command:\n"
"\n"
"  import file_read as SCOPE;\n"
"\n"
"where SCOPE is \047public\047 or \047private\047. This only effects the declarations\n"
"of the functions, not the placement of the typedef etc.\n"

#line 17 "help.u"
;
#line 81 "commands.u"

command_t commands[]=
{
#line 49 "commands.u"

{ cmd_sethmode, { fID, "headers", fPUN, ";", 0 } },
{ cmd_setcmode, { fID, "code", fPUN, ";", 0 } },
{ cmd_setbmode, { fID, "headers_and_code", fPUN, ";", 0 } },
#line 98 "commands.u"

 {cmd_stack, {fID, 0, fPUN, ":", fID, "stack", fROUND_ANY, fPUN, ";", 0}},

#line 447 "tree.u"

{ cmd_tree, { fID, 0, fPUN, ":", fID, "tree",
              fROUND_ARGS, (void*) 2,
              fID, "access", fCURLY_ANY,
              fID, "compare",fCURLY_ANY, 
              fPUN, ";", 0}},

#line 19 "qsort.u"

 { cmd_qsort, fID, "quick_sort",
      fPUN, "(",
          fANY,
             fPUN, ",",
          fANY,
             fPUN, ",",
          fANY,
      fPUN, ")",
     fCURLY_ANY, 0 },

#line 96 "buffer.u"
{ cmd_buffer, { fID, 0, fPUN, ":", fID, "buffer",
              fROUND_ARGS, (void*) 2,
              fPUN, ";", 0}},

#line 34 "for_words.u"

{ cmd_for_words, fID, "for_words", fID, 0, fROUND_ANY, fCURLY_ANY, 0 },

#line 34 "unify.u"

 { cmd_unify,
     fID, "unify",
     fROUND_ARGS, (void*) 3,
     fCURLY_ANY, 0 },

#line 310 "ex_split_words.u"

 { cmd_splitwords, { fID, "import", fID, "split_words", fID, "as", 
                     fID, 0, fPUN, ";", 0 }},

#line 105 "ex_file.u"

 { cmd_fileread, { fID, "import", fID, "file_read", fID, "as", 
                     fID, 0, fPUN, ";", 0 }},

#line 85 "commands.u"
  { 0, { 0, 0 } }
};

#line 66 "commands.u"
void cmd_setcmode(char **V) { 
  omode= 'C'; 
}
#line 70 "commands.u"
void cmd_sethmode(char **V) { 
  omode= 'H'; 
}
#line 74 "commands.u"
void cmd_setbmode(char **V) { 
  omode= 'H' +'C';
}
#line 141 "commands.u"
static int isidbegin(unsigned int chr)
{
  return isalnum(chr) || chr=='_';
}

#line 147 "commands.u"
static int isid(unsigned int chr)
{
  return isidbegin(chr) || isdigit(chr);
}

#line 205 "commands.u"
static int skip_quote(char **inp)
{
  char* IN= *inp;
  int q= *(IN++);
  while(*IN) {
     if (*IN=='\\') { IN++; if (!*IN) return 1; IN++; }
     else if (*IN==q) { *inp= IN+1; return 0; }
     else IN++;
  }
  return 1;
}

#line 218 "commands.u"
static char* collect_str(char *b,char *e)
{
   char *R= malloc(e-b+1);
   memcpy(R,b, e-b);
   R[e-b]= 0;
   return R;
}

#line 230 "commands.u"
void parse_string(char* str)
{
   void *ostack_storage[100];
   void **ptable;
   char *inp;
   void **ostack;
   int i;

   while(1)
   {
     while(isspace(*str)) str++;
     if (!*str) break;
     for(i=0;commands[i].func;i++)
     {
       int E=0;
       inp= str;
       ostack= ostack_storage;
       ptable= commands[i].parse_table;
       while(*ptable) {
         int (*F)(void ***,char**, void***)= *(ptable++);
         if (E=(*F)(&ptable, &inp, &ostack))
            break;
       }
       if (!E) {
         void (*F)(char**)= commands[i].func;
         (*F)((char**) ostack_storage);
         str= inp;
         break;
       }
     }
     if (!commands[i].func) {
        fprintf(stderr,"syntax error here: ");
        while(*str && *str!='\n') fprintf(stderr,"%c", *(str++));
        exit(fprintf(stderr,"\n"));
     }
   }
}
 
#line 272 "commands.u"

void cmd_stack(char **V)
{
   char *target= V[0];
   char *base= V[1];
   if (omode=='H' || omode=='C'+'H') {
#line 391 "commands.u"

prt(
#line 554 "t2c-ar.c"
"\n"
"typedef struct {\n"
"  int selts;\n"
"  int tos;\n"
"  BASE *elts; \n"
"} STACK_t;\n"
"\n"
"STACK_t *STACK_new(int initial);\n"
"int STACK_empty(STACK_t *S);\n"
"void STACK_push(STACK_t *S, BASE V);\n"
"int STACK_pop(STACK_t *S, BASE* R);\n"
"\n"

#line 396 "commands.u"
, "STACK", target, "BASE", base, 0);

#line 279 "commands.u"
   }
   if (omode=='C' || omode=='C'+'H') {
#line 399 "commands.u"
prt(
#line 575 "t2c-ar.c"
"\n"
"STACK_t *STACK_new(int initial)\n"
"{\n"
"  STACK_t *S;\n"
"  S= malloc(sizeof(*S));\n"
"  S->selts= initial;\n"
"  S->tos= -1;\n"
"  S->elts= malloc(sizeof(BASE)*S->selts); \n"
"  return S;\n"
"}\n"
"\n"
"int STACK_empty(STACK_t *S)\n"
"  { return S->tos==-1; }\n"
"\n"
"void STACK_push(STACK_t *S, BASE V)\n"
"{\n"
"  if(S->tos==S->selts-1) {\n"
"    S->selts+= S->selts/2;\n"
"    S->elts= realloc(S->elts, sizeof(BASE)*S->selts);\n"
"  }\n"
"  S->elts[++S->tos]= V;\n"
"}\n"
"\n"
"int STACK_pop(STACK_t *S, BASE* R)\n"
"{\n"
"  if (S->tos==-1) return 0;\n"
"  if (R) *R= S->elts[S->tos];\n"
"  S->tos--;\n"
"  return 1;\n"
"}\n"
"\n"

#line 403 "commands.u"
, "STACK", target, "BASE", base, 0);

#line 282 "commands.u"
   }
}

#line 512 "tree.u"

void cmd_tree(char **V)
{
  if (omode=='H' || omode=='H'+'C') 
   prt(
#line 620 "t2c-ar.c"
"\n"
"typedef struct TREE_s TREE_t;\n"
"\n"
"\n"
"TREE_t *TREE_new();\n"
"int TREE_find(TREE_t *T, KEY key, DATA *value);\n"
"int TREE_insert(TREE_t *T, DATA data) ;\n"
"int TREE_remove(TREE_t *T, KEY key, DATA *value);\n"
"int TREE_destroy(TREE_t *T,DATA *value);\n"
"\n"

#line 520 "tree.u"
      , "TREE", V[0], "KEY", V[1], "DATA", V[2],
        "ACCESS", V[3], "COMPARE", V[4], 0);
  if (omode=='C' || omode=='H'+'C') 
   prt(
#line 636 "t2c-ar.c"
"\n"
"\n"
"typedef struct TREE_node {\n"
"   struct TREE_node *left, *right,*parent;\n"
"   int height;\n"
"   DATA data;\n"
"} TREE_node_t;\n"
"\n"
"struct TREE_s {\n"
"  TREE_node_t *root;\n"
"};\n"
"\n"
"\n"
"TREE_node_t *TREE_find_node(TREE_t *T, KEY key2);\n"
"static void TREE_rebalance(TREE_t *T, TREE_node_t *N);\n"
"  \n"
"static void TREE_rotate_left(TREE_t *T, TREE_node_t *A);\n"
"static void TREE_rotate_left_twolevels(TREE_t *T, TREE_node_t *A);\n"
"static void TREE_rotate_right(TREE_t *T, TREE_node_t *A);\n"
"static void TREE_rotate_right_twolevels(TREE_t *T, TREE_node_t *A);\n"
"static int TREE_right_heavy(TREE_node_t *A);\n"
"static int TREE_left_heavy(TREE_node_t *A);\n"
"static void TREE_remove_node(TREE_t *T, TREE_node_t *N);\n"
"static TREE_node_t *TREE_remove_max(TREE_t *T,TREE_node_t *N);\n"
"static TREE_node_t *TREE_remove_min(TREE_t *T,TREE_node_t *N);\n"
"static void TREE_calculate_height(TREE_node_t *N);\n"
"static void TREE_replace_child_in_parent\n"
"    (TREE_t* T, TREE_node_t* old, TREE_node_t* nw);\n"
"static void TREE_set_left_child(TREE_node_t *P, TREE_node_t *N);\n"
"static void TREE_set_right_child(TREE_node_t *P, TREE_node_t *N);\n"
"static int TREE_height(TREE_node_t * A);\n"
"\n"
"TREE_node_t *TREE_find_node(TREE_t *T, KEY key2)\n"
"{\n"
"   TREE_node_t *node;\n"
"   node= T->root;\n"
"   while(node) {\n"
"     int result;\n"
"     DATA data;\n"
"     KEY key; KEY key1;\n"
"     data= node->data;\n"
"     { ACCESS }\n"
"     key1= key;\n"
"     { COMPARE  }\n"
"     if (result==0) return node;\n"
"     else if (result<0) node= node->left;\n"
"     else node= node->right;\n"
"   }\n"
"   return 0;\n"
"}\n"
"static void TREE_rebalance(TREE_t *T, TREE_node_t *N)\n"
"{\n"
"  int u;\n"
"  TREE_node_t *P;\n"
"\n"
"  TREE_calculate_height(N);\n"
"\n"
"  u = TREE_height(N->left)-TREE_height(N->right);\n"
"  P = N->parent;\n"
"  if (u < -1) {\n"
"      if (TREE_left_heavy(N->right)) TREE_rotate_left_twolevels(T, N);\n"
"      else TREE_rotate_left(T, N);\n"
"  } else if (u > 1) {\n"
"      if (TREE_right_heavy(N->left)) TREE_rotate_right_twolevels(T, N);\n"
"      else TREE_rotate_right(T, N);\n"
"  }\n"
"\n"
"  if (P) TREE_rebalance(T, P);\n"
"}\n"
"\n"
"  \n"
"static void TREE_rotate_left(TREE_t *T, TREE_node_t *A)\n"
"{\n"
"  TREE_node_t *B;\n"
"  B = A->right;\n"
"  TREE_set_right_child(A, B->left);\n"
"  TREE_replace_child_in_parent(T, A, B);\n"
"  TREE_set_left_child(B, A);\n"
"  TREE_calculate_height(A);\n"
"  TREE_calculate_height(B);\n"
"}\n"
"static void TREE_rotate_left_twolevels(TREE_t *T, TREE_node_t *A)\n"
"{\n"
"  TREE_node_t *B, *C;\n"
"\n"
"  B = A->right; C = B->left;\n"
"  TREE_set_right_child(A, C->left);\n"
"  TREE_set_left_child(B, C->right);\n"
"  TREE_replace_child_in_parent(T, A, C);\n"
"  TREE_set_left_child(C, A);\n"
"  TREE_set_right_child(C, B);\n"
"  TREE_calculate_height(A);\n"
"  TREE_calculate_height(B);\n"
"  TREE_calculate_height(C);\n"
"}\n"
"\n"
"static void TREE_rotate_right(TREE_t *T, TREE_node_t *A)\n"
"{\n"
"  TREE_node_t *B;\n"
"\n"
"  B = A->left;\n"
"\n"
"  A->left = B->right;\n"
"  if (A->left) A->left->parent = A;\n"
"\n"
"  B->parent = A->parent;\n"
"  TREE_replace_child_in_parent(T, A, B);\n"
"\n"
"  B->right = A;\n"
"  A->parent = B;\n"
"\n"
"  TREE_calculate_height(A);\n"
"  TREE_calculate_height(B);\n"
"}\n"
"\n"
"static void TREE_rotate_right_twolevels(TREE_t *T, TREE_node_t *A)\n"
"{\n"
"  TREE_node_t *B, *C;\n"
"\n"
"  B = A->left;\n"
"  C = B->right;\n"
"  /* we know that B and C are non-null since\n"
"     A was left heavy and B was right heavy */\n"
"\n"
"  A->left = C->right;\n"
"  if (A->left) A->left->parent = A;\n"
"\n"
"  B->right = C->left;\n"
"  if (B->right) B->right->parent = B;\n"
"\n"
"  C->parent = A->parent;\n"
"  TREE_replace_child_in_parent(T, A, C);\n"
"\n"
"  C->left = B;\n"
"  C->right = A;\n"
"  A->parent = C;\n"
"  B->parent = C;\n"
"\n"
"  TREE_calculate_height(A);\n"
"  TREE_calculate_height(B);\n"
"  TREE_calculate_height(C);\n"
"}\n"
"static int TREE_right_heavy(TREE_node_t *A)\n"
"{\n"
"  if (!A) return 0;\n"
"  return (TREE_height(A->right)-TREE_height(A->left)) > 0;\n"
"}\n"
"\n"
"static int TREE_left_heavy(TREE_node_t *A)\n"
"{\n"
"  if (!A) return 0;\n"
"  return (TREE_height(A->right)-TREE_height(A->left)) < 0;\n"
"}\n"
"\n"
"static void TREE_remove_node(TREE_t *T, TREE_node_t *N)\n"
"{\n"
"   TREE_node_t *Z;\n"
"   if (!N->left && !N->right) {\n"
"     TREE_replace_child_in_parent(T, N, 0);\n"
"     if (N->parent) TREE_rebalance(T,N->parent);\n"
"   } else {\n"
"     if (TREE_left_heavy(N)) Z= TREE_remove_max(T,N->left);\n"
"     else Z= TREE_remove_min(T,N->right);\n"
"     TREE_replace_child_in_parent(T, N, Z);\n"
"     TREE_set_left_child(Z, N->left);\n"
"     TREE_set_right_child(Z, N->right);\n"
"     TREE_rebalance(T,Z);\n"
"   }\n"
"   free(N);\n"
"}\n"
"static TREE_node_t *TREE_remove_max(TREE_t *T,TREE_node_t *N)\n"
"{\n"
"   while(N->right) N= N->right;\n"
"   TREE_replace_child_in_parent(T, N, N->left);\n"
"   if (N->parent) TREE_rebalance(T,N->parent);\n"
"   return N;\n"
"}\n"
"\n"
"static TREE_node_t *TREE_remove_min(TREE_t *T,TREE_node_t *N)\n"
"{\n"
"   while(N->left) N= N->left;\n"
"   TREE_replace_child_in_parent(T, N, N->right);\n"
"   if (N->parent) TREE_rebalance(T,N->parent);\n"
"   return N;\n"
"}\n"
"\n"
"static void TREE_calculate_height(TREE_node_t *N)\n"
"{\n"
"  int lH, rH;\n"
"  lH= TREE_height(N->left);\n"
"  rH= TREE_height(N->right);\n"
"  if (lH>rH) N->height= lH+1; else N->height= rH+1;\n"
"}\n"
"\n"
"static void TREE_replace_child_in_parent\n"
"    (TREE_t* T, TREE_node_t* old, TREE_node_t* nw)\n"
"{\n"
"  if (old->parent)\n"
"    {\n"
"      if (old->parent->left == old)\n"
"\011old->parent->left = nw;\n"
"      else\n"
"\011old->parent->right = nw;\n"
"    }\n"
"  else\n"
"    {\n"
"      T->root = nw;\n"
"    }\n"
"  if (nw) nw->parent= old->parent;\n"
"}\n"
"\n"
"static void TREE_set_left_child(TREE_node_t *P, TREE_node_t *N)\n"
"{\n"
"  P->left= N; \n"
"  if (N) N->parent= P;\n"
"}\n"
"\n"
"static void TREE_set_right_child(TREE_node_t *P, TREE_node_t *N)\n"
"{\n"
"  P->right= N; \n"
"  if (N) N->parent= P;\n"
"}\n"
"\n"
"static int TREE_height(TREE_node_t * A)\n"
"{\n"
"  if (A) return A->height;\n"
"  else return 0;\n"
"}\n"
"\n"
"\n"
"TREE_t *TREE_new()\n"
"{\n"
"  TREE_t *T;\n"
"  T= malloc(sizeof(*T));\n"
"  T->root= 0;\n"
"  return T;\n"
"}\n"
"\n"
"\n"
"int TREE_find(TREE_t *T, KEY key, DATA *value)\n"
"{\n"
"   TREE_node_t *n;\n"
"   n= TREE_find_node(T, key);\n"
"   if (n) { if (value) *value= n->data; return 1; }\n"
"   else return 0;\n"
"}\n"
"\n"
"int TREE_insert(TREE_t *T, DATA data) \n"
"{\n"
"  TREE_node_t *N,*P,*K,**PL;\n"
"  KEY key; KEY key2; KEY key1;\n"
"  int result;\n"
"\n"
"  N = calloc(1, sizeof(*N));\n"
"  N->data= data;\n"
"  N->height= 1;\n"
"\n"
"  P = 0;\n"
"  PL= &T->root;\n"
"  K= T->root;\n"
"  { ACCESS }\n"
"  key2= key;\n"
"  while (K) {\n"
"    data= K->data;\n"
"    { ACCESS }\n"
"    key1= key;\n"
"    { COMPARE }\n"
"    if (result<0)      { PL= &K->left; P= K; K= K->left; }\n"
"    else if (result>0) { PL= &K->right; P= K; K= K->right; }\n"
"    else               { free(N); return 0; }\n"
"  }\n"
"  *PL= N;\n"
"  N->parent= P;\n"
"  TREE_rebalance(T, N);\n"
"  return 1;\n"
"}\n"
"int TREE_remove(TREE_t *T, KEY key, DATA *value)\n"
"{\n"
"   TREE_node_t *N;\n"
"   N= TREE_find_node(T,key);\n"
"   if (!N) return 0;\n"
"   if (value) *value= N->data;\n"
"   TREE_remove_node(T,N);\n"
"   return 1;\n"
"}\n"
"int TREE_destroy(TREE_t *T,DATA *value)\n"
"{\n"
"   TREE_node_t *N= T->root;\n"
"   if (!N) return 0;\n"
"   while(N->left) N=N->left;\n"
"   if (value) *value= N->data;\n"
"   TREE_remove_node(T,N);\n"
"   return 1;\n"
"}\n"
"\n"

#line 527 "tree.u"
      , "TREE", V[0], "KEY", V[1], "DATA", V[2],
        "ACCESS", V[3], "COMPARE", V[4], 0);
}

#line 86 "qsort.u"

static int skip_paren(char **s)
{
  char *X= *s;
  int begin= *(X++), end; 
  char *pairs= "{}()[]";
  int k;
  for(;*pairs;pairs++) if (pairs[0]==begin) { end= pairs[1]; break; }
  k= 1;
  while(1)
  {
    if (*X=='\'' || *X=='\"') { if (skip_quote(&X)) return 1; }
    else if (*X==begin) { X++; k++; }
    else if (*X==end) { X++; k--; if (k==0) break; }
    else if (*X==0) { return 1; }
    else X++;
  }
  *s= X;
  return 0;
}

#line 243 "qsort.u"

void cmd_qsort(char **V)
{
#if 0
   printf("QuickSort (%s, %s, %s, %s)\n",
        V[0], V[1], V[2], V[3]);
#endif

   prt(
#line 969 "t2c-ar.c"
"{\n"
"\n"
" /* carchive.quick_sort\n"
"      type= QS_TYPE\n"
"      var = QS_VARNAME\n"
"      size= QS_SIZE\n"
"      compare= QS_COMPARE\n"
" */\n"
"\n"
"  int stack[72];\n"
"  int tos= -1;\n"
"  int left, QSP, ppos, QSI, QSRESULT;\n"
"  QS_TYPE temp;\n"
"\n"
"  left= 0;\n"
"  QSP= QS_SIZE - 1;\n"
"\n"
"  if (QSP<=left) goto done;\n"
"\n"
" again:\n"
" \n"
"  ppos= left;\n"
"\n"
"  for(QSI=left;QSI<QSP;QSI++)\n"
"   {\n"
"      { QS_COMPARE }\n"
"      if (QSRESULT<=0)\n"
"       {\n"
"         temp = QS_VARNAME[QSI];\n"
"         QS_VARNAME[QSI]= QS_VARNAME[ ppos ];\n"
"         QS_VARNAME[ppos]= temp;\n"
"         ppos++;\n"
"       }\n"
"   }\n"
"  temp= QS_VARNAME[QSP];\n"
"  QS_VARNAME[QSP]= QS_VARNAME[ppos];\n"
"  QS_VARNAME[ppos]= temp;\n"
"\n"
"  int left1, left2, right1, right2;\n"
"\n"
"  left1 = left;   right1= ppos-1;\n"
"  left2 = ppos+1; right2 = QSP;\n"
"\n"
"  if (right1-left1 < right2-left2)\n"
"  {\n"
"    stack[++tos]= left2; stack[++tos]= right2;\n"
"    stack[++tos]= left1; stack[++tos]= right1;\n"
"  } else {\n"
"    stack[++tos]= left1; stack[++tos]= right1;\n"
"    stack[++tos]= left2; stack[++tos]= right2;\n"
"  }\n"
"\n"
"  while(tos!=-1) {\n"
"    QSP= stack[tos--];\n"
"    left= stack[tos--];\n"
"    if (left>=QSP) continue;\n"
"    goto again;\n"
"  }\n"
"done:\n"
"  left= QSP;  /* so that gcc doesn\047t warn us about a label not preceding\n"
"                 a statement and such. */\n"
"\n"
"}\n"

#line 257 "qsort.u"
   , 
#line 222 "qsort.u"

  "stack", gensymbol(),
  "left1", gensymbol(),
  "left2", gensymbol(),
  "right1", gensymbol(),
  "right2", gensymbol(),
  "left",  gensymbol(),
  "done",  gensymbol(),
  "again",  gensymbol(),
  "temp",  gensymbol(),
  "tos",   gensymbol(),
  "ppos",   gensymbol(),

  "QS_TYPE", V[0],
  "QS_VARNAME", V[1],
  "QS_SIZE", V[2],
  "QS_COMPARE", V[3],
  
  

#line 259 "qsort.u"
   0);
}

#line 281 "qsort.u"

static char *gensymbol()
{
   char buf[200];
   static int last= 1;
   sprintf(buf, "yyqr%d", last++);
   return strdup(buf);
}

#line 123 "buffer.u"

static void cmd_buffer(char **V)
{
#line 105 "buffer.u"

  char *prefix= strdup(V[0]);
  int L= strlen(prefix);
  if (L>2 && prefix[L-1]=='t' && prefix[L-2]=='_')
    prefix[L-2]= 0;

#line 127 "buffer.u"
  if (omode=='H' || omode=='H'+'C') 
   prt(
#line 1084 "t2c-ar.c"
"\n"
"#ifndef _buffer_type_BUFFER_defined\n"
"#define _buffer_type_BUFFER_defined\n"
"\n"
"typedef struct {\n"
"    struct BUFFER_node *first, *last;\n"
"} BUFFER;\n"
"#endif\n"
"\n"
"\n"
"BUFFER *PREFIX_new();\n"
"void PREFIX_append(BUFFER *B, TYPE d);\n"
"TYPE *PREFIX_collect(BUFFER *B, int *len);\n"
"TYPE *PREFIX_collectd(BUFFER *B, int *len);\n"
"void PREFIX_free(BUFFER *B);\n"

#line 135 "buffer.u"
    ,
#line 112 "buffer.u"

 "BUFFER", V[0],
 "PREFIX", prefix,
 "TYPE", V[1],
 "BLOCKSIZE", V[2],
 NULL

#line 137 "buffer.u"
   );

  if (omode=='C' || omode=='H'+'C') 
   prt(
#line 1115 "t2c-ar.c"
"\n"
"#ifndef _buffer_type_BUFFER_defined\n"
"#define _buffer_type_BUFFER_defined\n"
"\n"
"typedef struct {\n"
"    struct BUFFER_node *first, *last;\n"
"} BUFFER;\n"
"#endif\n"
"\n"
"\n"
"struct BUFFER_node {\n"
"   struct BUFFER_node *next;\n"
"   int len;\n"
"   TYPE data[BLOCKSIZE];\n"
"};\n"
"\n"
"\n"
"static struct BUFFER_node *BUFFER_newblock();\n"
"BUFFER *PREFIX_new();\n"
"void PREFIX_append(BUFFER *B, TYPE d);\n"
"TYPE *PREFIX_collect(BUFFER *B, int *len);\n"
"TYPE *PREFIX_collectd(BUFFER *B, int *len);\n"
"void PREFIX_free(BUFFER *B);\n"
"\n"
"static struct BUFFER_node *BUFFER_newblock()\n"
"{\n"
"  struct BUFFER_node *R= malloc(sizeof(*R));\n"
"  R->len= 0;\n"
"  R->next= 0;\n"
"  return R;\n"
"}\n"
"\n"
"BUFFER *PREFIX_new()\n"
"{\n"
"   BUFFER *B= malloc(sizeof(*B));\n"
"   B->first= B->last= BUFFER_newblock();\n"
"   return B;\n"
"}\n"
"\n"
"void PREFIX_append(BUFFER *B, TYPE d)\n"
"{\n"
"   struct BUFFER_node *L= B->last;\n"
"   if (L->len==BLOCKSIZE)\n"
"     B->last= L= B->last->next= BUFFER_newblock();\n"
"   L->data[L->len++]= d;\n"
"}\n"
"\n"
"TYPE *PREFIX_collect(BUFFER *B, int *len)\n"
"{\n"
"   int S= 0;\n"
"   struct BUFFER_node *N;\n"
"   TYPE *R;\n"
"   for(N=B->first;N;N=N->next)\n"
"     S+= N->len;\n"
"   if (S) {\n"
"     R= malloc(S*sizeof(TYPE));\n"
"     S= 0;\n"
"     for(N=B->first;N;N=N->next)\n"
"       { memcpy(R+S, N->data, sizeof(TYPE)*N->len);\n"
"         S+= N->len; }\n"
"   } else R= 0;\n"
"   if (len) *len= S;\n"
"   return R;\n"
"}\n"
"\n"
"TYPE *PREFIX_collectd(BUFFER *B, int *len)\n"
"{\n"
"   TYPE *R;\n"
"   R= PREFIX_collect(B, len);\n"
"   PREFIX_free(B);\n"
"   return R;\n"
"}\n"
"\n"
"void PREFIX_free(BUFFER *B)\n"
"{\n"
"   struct BUFFER_node *N,*T;\n"
"   for(N=B->first;N;N=T)\n"
"    { T= N->next; free(N); }\n"
"   free(B);\n"
"}\n"
"\n"

#line 151 "buffer.u"
    ,
#line 112 "buffer.u"

 "BUFFER", V[0],
 "PREFIX", prefix,
 "TYPE", V[1],
 "BLOCKSIZE", V[2],
 NULL

#line 153 "buffer.u"
   );

}

#line 42 "for_words.u"

void cmd_for_words(char **V)
{
  char *loop_var= V[0];
  char *word_text= V[1];
  char *body= V[2];
  int i;

  char **words= split_words(word_text);
  for(i=0;words[i];i++)
  {
    prt(body, loop_var, words[i],0);
  }
  free(words);

}

#line 55 "unify.u"

void cmd_unify(char **V)
{
  prt(
#line 1235 "t2c-ar.c"
"\n"
" { /* unification code generated by t2c.ar\n"
"      parameters:\n"
"        array= ARRAY \n"
"        length= SIZE\n"
"        final length in VAR\n"
"        compare= COMPARE */\n"
"   int UAB=0, UAA= 0;\n"
"   int UAR, UA_N= SIZE;\n"
"   VAR= 0;\n"
"   while(UAA<UA_N && UAB<UA_N)\n"
"   {\n"
"     COMPARE\n"
"     if (UAR)\n"
"     {\n"
"       UAA++;\n"
"       if (UAA!=UAB) ARRAY[UAA]= ARRAY[UAB];\n"
"     }\n"
"     UAB++;\n"
"   }\n"
"   if (UA_N) VAR= UAA+1;\n"
"   else VAR= 0;\n"
" }\n"
" \n"

#line 62 "unify.u"
  ,
#line 45 "unify.u"

  "UA_N", gensymbol(),
  "UA_CUR", gensymbol(),
  "SIZE", V[1],
  "ARRAY", V[0],
  "VAR", V[2],
  "COMPARE", V[3],
  
#line 64 "unify.u"
  0);
}

#line 24 "help.u"

static char *get_next_section(char *start,char *secname)
{
  char *k;
  char *Z;
  *secname= 0;
  k= strstr(start,"\n[[");
  if (!k) return 0;
  for(Z=k+3;*Z!=']'&& *Z!=0; Z++)
    *(secname++)= *Z;
  *secname= 0; 
  if (*Z) Z++;
  return Z;
}

#line 43 "help.u"
static void print_help_toc()
{
  char *start;
  char secname[2000];
  printf("Contents\n");
  for(start=help;start;start=get_next_section(start,secname))
     if (secname[0]) printf("  %s\n", secname);
}

#line 56 "help.u"

static void do_help(int argc,char **argv)
{
  if (!strcmp(argv[0], "toc")) { print_help_toc(); exit(0); }
  else print_man(argv[0]);
}

#line 67 "help.u"
static void print_man(char *name)
{
  char *start;
  char *next;
  start= get_section(name);
  if (!start) { printf("can't find section %s\n", name); exit(1); }

  next= strstr(start,"\n[[");
  if (!next) next= start+strlen(start);

  printf("Manual for: %s\n\n", name);

  while(*start=='\n') start++;

  fwrite(start, 1, next-start, stdout);
  exit(0);
}

#line 86 "help.u"

static char *get_section(char *name)
{
  char buf[2000];
  char *Z;
  sprintf(buf,"[[ %s ]]", name);
  Z= strstr(help, buf);
  if (!Z) return 0;
  return Z+strlen(buf);
}
#line 318 "ex_split_words.u"

static void cmd_splitwords(char **V)
{
  char *scope;
  if (!strcmp(V[0],"public")) scope="";
  else if (!strcmp(V[0],"private")) scope= "static";
  else exit(fprintf(stderr,"unknown scope in import split_words : %s\n",
                        V[0]));
  if (omode=='H' || omode=='H'+'C')
  prt(
#line 1350 "t2c-ar.c"
"\n"
"SCOPE char **split_words(char *src);\n"
"\n"

#line 331 "ex_split_words.u"
  , "SCOPE", scope,0);
  if (omode=='C' || omode=='H'+'C') 
   prt(
#line 1358 "t2c-ar.c"
"\n"
"SCOPE char **split_words(char *src)\n"
"{\n"
"  int for_real= 0;\n"
"  char **ret;\n"
"  char *str_area;\n"
"  int nwords=0;\n"
"  char *cursor, *marker,*start;\n"
"\n"
"  cursor= src;\n"
"  start= cursor;\n"
"\n"
"again:\n"
"\n"
"#line 38 \042words.c\042\n"
"{\n"
"\011char yych;\n"
"\n"
"\011yych = *cursor;\n"
"\011switch (yych) {\n"
"\011case 0x00:\011goto yy10;\n"
"\011case 0x01:\n"
"\011case 0x02:\n"
"\011case 0x03:\n"
"\011case 0x04:\n"
"\011case 0x05:\n"
"\011case 0x06:\n"
"\011case 0x07:\n"
"\011case 0x08:\n"
"\011case \047\134t\047:\n"
"\011case \047\134n\047:\n"
"\011case \047\134v\047:\n"
"\011case \047\134f\047:\n"
"\011case \047\134r\047:\n"
"\011case 0x0E:\n"
"\011case 0x0F:\n"
"\011case 0x10:\n"
"\011case 0x11:\n"
"\011case 0x12:\n"
"\011case 0x13:\n"
"\011case 0x14:\n"
"\011case 0x15:\n"
"\011case 0x16:\n"
"\011case 0x17:\n"
"\011case 0x18:\n"
"\011case 0x19:\n"
"\011case 0x1A:\n"
"\011case 0x1B:\n"
"\011case 0x1C:\n"
"\011case 0x1D:\n"
"\011case 0x1E:\n"
"\011case 0x1F:\n"
"\011case \047 \047:\n"
"\011case 0x7F:\011goto yy2;\n"
"\011case \047\042\047:\011goto yy5;\n"
"\011default:\011goto yy8;\n"
"\011}\n"
"yy2:\n"
"\011++cursor;\n"
"\011yych = *cursor;\n"
"\011switch (yych) {\n"
"\011case 0x01:\n"
"\011case 0x02:\n"
"\011case 0x03:\n"
"\011case 0x04:\n"
"\011case 0x05:\n"
"\011case 0x06:\n"
"\011case 0x07:\n"
"\011case 0x08:\n"
"\011case \047\134t\047:\n"
"\011case \047\134n\047:\n"
"\011case \047\134v\047:\n"
"\011case \047\134f\047:\n"
"\011case \047\134r\047:\n"
"\011case 0x0E:\n"
"\011case 0x0F:\n"
"\011case 0x10:\n"
"\011case 0x11:\n"
"\011case 0x12:\n"
"\011case 0x13:\n"
"\011case 0x14:\n"
"\011case 0x15:\n"
"\011case 0x16:\n"
"\011case 0x17:\n"
"\011case 0x18:\n"
"\011case 0x19:\n"
"\011case 0x1A:\n"
"\011case 0x1B:\n"
"\011case 0x1C:\n"
"\011case 0x1D:\n"
"\011case 0x1E:\n"
"\011case 0x1F:\n"
"\011case \047 \047:\n"
"\011case 0x7F:\011goto yy2;\n"
"\011default:\011goto yy4;\n"
"\011}\n"
"yy4:\n"
"#line 40 \042words.cr\042\n"
"\011{ start= cursor; goto again; }\n"
"#line 123 \042words.c\042\n"
"yy5:\n"
"\011marker = ++cursor;\n"
"\011yych = *cursor;\n"
"\011switch (yych) {\n"
"\011case 0x00:\011goto yy7;\n"
"\011case 0x01:\n"
"\011case 0x02:\n"
"\011case 0x03:\n"
"\011case 0x04:\n"
"\011case 0x05:\n"
"\011case 0x06:\n"
"\011case 0x07:\n"
"\011case 0x08:\n"
"\011case \047\134t\047:\n"
"\011case \047\134n\047:\n"
"\011case \047\134v\047:\n"
"\011case \047\134f\047:\n"
"\011case \047\134r\047:\n"
"\011case 0x0E:\n"
"\011case 0x0F:\n"
"\011case 0x10:\n"
"\011case 0x11:\n"
"\011case 0x12:\n"
"\011case 0x13:\n"
"\011case 0x14:\n"
"\011case 0x15:\n"
"\011case 0x16:\n"
"\011case 0x17:\n"
"\011case 0x18:\n"
"\011case 0x19:\n"
"\011case 0x1A:\n"
"\011case 0x1B:\n"
"\011case 0x1C:\n"
"\011case 0x1D:\n"
"\011case 0x1E:\n"
"\011case 0x1F:\n"
"\011case \047 \047:\n"
"\011case 0x7F:\011goto yy14;\n"
"\011case \047\042\047:\011goto yy12;\n"
"\011default:\011goto yy5;\n"
"\011}\n"
"yy7:\n"
"#line 42 \042words.cr\042\n"
"\011{ goto do_word; }\n"
"#line 168 \042words.c\042\n"
"yy8:\n"
"\011++cursor;\n"
"\011yych = *cursor;\n"
"\011switch (yych) {\n"
"\011case 0x00:\n"
"\011case 0x01:\n"
"\011case 0x02:\n"
"\011case 0x03:\n"
"\011case 0x04:\n"
"\011case 0x05:\n"
"\011case 0x06:\n"
"\011case 0x07:\n"
"\011case 0x08:\n"
"\011case \047\134t\047:\n"
"\011case \047\134n\047:\n"
"\011case \047\134v\047:\n"
"\011case \047\134f\047:\n"
"\011case \047\134r\047:\n"
"\011case 0x0E:\n"
"\011case 0x0F:\n"
"\011case 0x10:\n"
"\011case 0x11:\n"
"\011case 0x12:\n"
"\011case 0x13:\n"
"\011case 0x14:\n"
"\011case 0x15:\n"
"\011case 0x16:\n"
"\011case 0x17:\n"
"\011case 0x18:\n"
"\011case 0x19:\n"
"\011case 0x1A:\n"
"\011case 0x1B:\n"
"\011case 0x1C:\n"
"\011case 0x1D:\n"
"\011case 0x1E:\n"
"\011case 0x1F:\n"
"\011case \047 \047:\n"
"\011case 0x7F:\011goto yy7;\n"
"\011default:\011goto yy8;\n"
"\011}\n"
"yy10:\n"
"\011++cursor;\n"
"#line 43 \042words.cr\042\n"
"\011{ goto do_eof; }\n"
"#line 213 \042words.c\042\n"
"yy12:\n"
"\011++cursor;\n"
"\011switch ((yych = *cursor)) {\n"
"\011case 0x00:\n"
"\011case 0x01:\n"
"\011case 0x02:\n"
"\011case 0x03:\n"
"\011case 0x04:\n"
"\011case 0x05:\n"
"\011case 0x06:\n"
"\011case 0x07:\n"
"\011case 0x08:\n"
"\011case \047\134t\047:\n"
"\011case \047\134n\047:\n"
"\011case \047\134v\047:\n"
"\011case \047\134f\047:\n"
"\011case \047\134r\047:\n"
"\011case 0x0E:\n"
"\011case 0x0F:\n"
"\011case 0x10:\n"
"\011case 0x11:\n"
"\011case 0x12:\n"
"\011case 0x13:\n"
"\011case 0x14:\n"
"\011case 0x15:\n"
"\011case 0x16:\n"
"\011case 0x17:\n"
"\011case 0x18:\n"
"\011case 0x19:\n"
"\011case 0x1A:\n"
"\011case 0x1B:\n"
"\011case 0x1C:\n"
"\011case 0x1D:\n"
"\011case 0x1E:\n"
"\011case 0x1F:\n"
"\011case \047 \047:\n"
"\011case 0x7F:\011goto yy13;\n"
"\011default:\011goto yy8;\n"
"\011}\n"
"yy13:\n"
"#line 41 \042words.cr\042\n"
"\011{ goto do_word; }\n"
"#line 256 \042words.c\042\n"
"yy14:\n"
"\011++cursor;\n"
"\011yych = *cursor;\n"
"\011switch (yych) {\n"
"\011case 0x00:\011goto yy16;\n"
"\011case \047\042\047:\011goto yy17;\n"
"\011default:\011goto yy14;\n"
"\011}\n"
"yy16:\n"
"\011cursor = marker;\n"
"\011goto yy7;\n"
"yy17:\n"
"\011++cursor;\n"
"\011yych = *cursor;\n"
"\011goto yy13;\n"
"}\n"
"#line 44 \042words.cr\042\n"
"\n"
"do_word:\n"
"  if (for_real) {\n"
"    int L= cursor-start;\n"
"    memcpy(str_area, start, L);\n"
"    ret[nwords]= str_area;\n"
"    str_area[L]= 0;\n"
"    str_area+= L+1;\n"
"  } \n"
"  nwords++;\n"
"  start= cursor;\n"
"  goto again; \n"
"do_eof:\n"
"  if (for_real) { ret[nwords]= 0; return ret;}\n"
"  ret= (char**) malloc(nwords+strlen(src)+ sizeof(char*)*(nwords+1));\n"
"  str_area= (char*) ret + sizeof(char*)*(nwords+1);\n"
"  nwords= 0; for_real= 1;\n"
"  cursor= src; start= src;\n"
"  goto again;\n"
"}\n"
"\n"

#line 337 "ex_split_words.u"
  , "SCOPE", scope,0);
}
#line 110 "ex_file.u"
static void cmd_fileread(char **V)
{
  char *scope;
  if (!strcmp(V[0],"public")) scope="";
  else if (!strcmp(V[0],"private")) scope= "static";
  else exit(fprintf(stderr,"unknown scope in import file_read : %s\n",
                        V[0]));
  if (omode=='H' || omode=='H'+'C')
  prt(
#line 1643 "t2c-ar.c"
"\n"
"typedef struct {\n"
"  char *name;\n"
"  char *error;\n"
"  size_t size;\n"
"  unsigned char* contents;\n"
"} file_t;\n"
"\n"
"SCOPE file_t *file_read(char *name);\n"
"SCOPE file_t *file_read_descriptor(FILE *D,char *name);\n"
"\n"

#line 122 "ex_file.u"
  , "SCOPE", scope,0);
  if (omode=='C' || omode=='H'+'C') 
   prt(
#line 1659 "t2c-ar.c"
"\n"
"typedef struct buffer {\n"
"  struct buffer *next;\n"
"  int len;\n"
"  char data[504];\n"
"} buffer_t;\n"
"\n"
"static void do_read(FILE *D, file_t *F)\n"
"{\n"
"  buffer_t **tail;\n"
"  buffer_t *head;\n"
"  buffer_t *B,*N;\n"
"\n"
"  tail= &head;\n"
"  head= 0;\n"
"\n"
"  while(1) {\n"
"     int R;\n"
"     B= malloc(sizeof(*B));\n"
"     R= fread(B->data, 1, sizeof(B->data), D);\n"
"     if (R<=0) {\n"
"        free(B);\n"
"        if (feof(D)) break;\n"
"        F->error= strdup(\042read failed\042);\n"
"        goto free_all;\n"
"     }\n"
"     B->len= R;\n"
"     B->next= 0;\n"
"     *tail= B; \n"
"     tail = &B->next;\n"
"  }\n"
"\n"
"  int L=0;\n"
"  for(B=head;B;B=B->next)\n"
"    L+= B->len;\n"
"\n"
"  F->contents= malloc(L+1);\n"
"  L= 0;\n"
"  for(B=head;B;B=B->next) {\n"
"      memcpy(F->contents+L, B->data, B->len);\n"
"      L+= B->len;\n"
"  }\n"
"  F->contents[L]= 0;\n"
"  F->size= L;\n"
"\n"
"free_all:\n"
"  for(B= head; B; B=N)\n"
"  {\n"
"     N= B->next;\n"
"     free(B);\n"
"  }\n"
"}\n"
"\n"
"SCOPE file_t *file_read(char *name)\n"
"{\n"
"   file_t *F;\n"
"   FILE *D;\n"
"   F= newfile(name);\n"
"   D= fopen(name, \042r\042);\n"
"   if (!D) {\n"
"      char buf[2000];\n"
"      sprintf(buf, \042can not open file for reading: %s\042, strerror(errno));\n"
"      F->error= strdup(buf);\n"
"      return F;\n"
"   }\n"
"   do_read(D, F);\n"
"   fclose(D);\n"
"   return F;\n"
"}\n"
"\n"
"SCOPE file_t *file_read_descriptor(FILE *D,char *name)\n"
"{\n"
"   file_t *F;\n"
"   F= newfile(name);\n"
"   do_read(D, F);\n"
"   return F;\n"
"}\n"
"\n"
"void *_file_c_unused_funcs[]=\n"
"{\n"
"  file_read_descriptor,\n"
"  file_read,\n"
"};\n"

#line 128 "ex_file.u"
  , "SCOPE", scope,0);
}

#line 53 "file.u"

static file_t *newfile(char* name)
{
  file_t *F;
  F= calloc(1,sizeof(*F));
  F->name= strdup(name);
  return F;
}

typedef struct buffer {
  struct buffer *next;
  int len;
  char data[504];
} buffer_t;

static void do_read(FILE *D, file_t *F)
{
  buffer_t **tail;
  buffer_t *head;
  buffer_t *B,*N;

  tail= &head;
  head= 0;

  while(1) {
     int R;
     B= malloc(sizeof(*B));
     R= fread(B->data, 1, sizeof(B->data), D);
     if (R<=0) {
	free(B);
        if (feof(D)) break;
        F->error= strdup("read failed");
        goto free_all;
     }
     B->len= R;
     B->next= 0;
     *tail= B; 
     tail = &B->next;
  }

  int L=0;
  for(B=head;B;B=B->next)
    L+= B->len;

  F->contents= malloc(L+1);
  L= 0;
  for(B=head;B;B=B->next) {
      memcpy(F->contents+L, B->data, B->len);
      L+= B->len;
  }
  F->contents[L]= 0;
  F->size= L;

free_all:
  for(B= head; B; B=N)
  {
     N= B->next;
     free(B);
  }
}


#line 26 "file.u"

file_t *file_read(char *name)
{
   file_t *F;
   FILE *D;
   F= newfile(name);
   D= fopen(name, "r");
   if (!D) {
      char buf[2000];
      sprintf(buf, "can not open file for reading: %s", strerror(errno));
      F->error= strdup(buf);
      return F;
   }
   do_read(D, F);
   fclose(D);
   return F;
}

file_t *file_read_descriptor(FILE *D,char *name)
{
   file_t *F;
   F= newfile(name);
   do_read(D, F);
   return F;
}

#line 344 "commands.u"

void prt(char *tm, ...)
{
  char *Z;
  va_list ap;
  va_start(ap, tm);
  Z= strdup(tm);
  prtv(Z, ap);
  free(Z);
  va_end(ap);
}

#line 357 "commands.u"

void prtv(char *tm, va_list ap)
{
  char *F, *R,*P;
  va_list apc;
  va_copy(apc, ap);

  F= va_arg(apc, char*);
  if (!F) { printf("%s", tm); va_end(apc); return ; }
  R= va_arg(apc, char*);

  P= strstr(tm, F);
  if (P) {
    *P= 0;
    prtv(tm, apc);
    printf("%s", R);
    prtv(P+strlen(F), ap);
  } else {
      // continue with the next replacement
    prtv(tm, apc);
  }
  va_end(apc);
}

#line 314 "commands.u"


int main(int argc,char **argv)
{
  file_t *f;
  if (argc>1) do_help(argc-1,argv+1);
  f= file_read_descriptor(stdin, "(stdin)");
  if (f->error) exit(fprintf(stderr,"file error: %s\n",f->error));
  parse_string(f->contents); 
  return 0;
}

